package cn.ibizlab.codegen.model;

import net.ibizsys.model.IPSSystem;
import net.ibizsys.model.PSModelServiceImpl;
import net.ibizsys.model.database.IPSDEFDTColumn;
import net.ibizsys.model.dataentity.IPSDataEntity;
import net.ibizsys.model.dataentity.defield.IPSDEField;
import net.ibizsys.model.dataentity.defield.IPSLinkDEField;
import net.ibizsys.model.dataentity.der.IPSDER1N;
import net.ibizsys.model.dataentity.der.IPSDERBase;
import net.ibizsys.model.dataentity.ds.IPSDEDataQueryCode;
import net.ibizsys.model.dataentity.ds.IPSDEDataQueryCodeCond;
import net.ibizsys.model.dataentity.ds.IPSDEDataSetGroupParam;
import net.ibizsys.model.dataentity.ds.PSDEDataSetGroupParamImpl;
import org.springframework.util.Assert;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DynamicModelStorage {

    private static DynamicModelStorage instance=null;

    public static DynamicModelStorage getInstance()
    {
        if(instance==null)
            instance=new DynamicModelStorage();
        return instance;
    }


    private String modelPath;

    public String getModelPath()
    {
        return modelPath;
    }

    public DynamicModelStorage modelPath(String modelPath)
    {
        this.modelPath=modelPath;
        return this;
    }

    private IPSSystem iPSSystem = null;

    public IPSSystem getPSSystem()
    {
        if(iPSSystem==null)
        {
            String strPSModelFolderPath=null;
            if(Files.exists(Paths.get(getModelPath(),"PSSYSTEM.json"))) {
                strPSModelFolderPath= Paths.get(getModelPath()).toString();
            }
            else if(Files.exists(Paths.get(getModelPath(),"model","PSSYSTEM.json"))) {
                strPSModelFolderPath= Paths.get(getModelPath(),"model").toString();
            }


            Assert.hasLength(strPSModelFolderPath,"加载系统模型错误,未找到对应模型目录");

            PSModelServiceImpl psModelService = new PSModelServiceImpl();
            psModelService.setPSModelFolderPath(strPSModelFolderPath);
            try {
                iPSSystem = psModelService.getPSSystem();
                Assert.notNull(iPSSystem,"加载系统模型错误:"+strPSModelFolderPath);


            } catch (Exception e) {
                throw new RuntimeException("加载系统模型错误"+strPSModelFolderPath,e);
            }
        }
        return iPSSystem;
    }



    private Map<String,EntityModel> entities=new HashMap<>();

    public  EntityModel getDynamicEntity(String entity) throws Exception
    {
        if(entities.containsKey(entity))
            return entities.get(entity);
        IPSSystem iPSSystem=getPSSystem();
        IPSDataEntity dataEntity = iPSSystem.getPSDataEntity(entity,true);
        Assert.notNull(dataEntity,"未找到对应的实体模型:"+entity);
        EntityModel entityModel=new EntityModel();
        entityModel.setStorageMode(dataEntity.getStorageMode());
        entityModel.setEntity(dataEntity);
        List<String> dsTypes=new ArrayList<>();

        if(dataEntity.getAllPSDEDBConfigs()!=null)
        {
            dataEntity.getAllPSDEDBConfigs().forEach(item->{
                dsTypes.add(item.getDBType().toLowerCase().replace("mysql5","mysql"));
                if(!entityModel.getTableName().equalsIgnoreCase(item.getTableName()))
                    entityModel.set("table-"+item.getDBType().toLowerCase().replace("mysql5","mysql"),item.getTableName());
            });
        }

        if(dsTypes.size()>0)
            entityModel.set("ds_types", String.join(",",dsTypes));


        if(dataEntity.getMinorPSDERs()!=null)
        {
            for(IPSDERBase der : dataEntity.getMinorPSDERs())
            {
                RelationshipModel rel=new RelationshipModel();
                rel.setRelation(der);
                rel.setName(der.getName()).setRelationType(der.getDERType()).setCodeName(der.getCodeName()).setEntityId(der.getMajorPSDataEntity().getId())
                        .setEntityCodeName(der.getMajorPSDataEntity().getCodeName()).setEntityName(der.getMajorPSDataEntity().getName())
                        .setEntityLogicName(der.getMajorPSDataEntity().getLogicName()).setTableName(der.getMajorPSDataEntity().getTableName());



                if(der instanceof IPSDER1N)
                {
                    IPSDER1N der1n=(IPSDER1N)der;
                    String relfieldname=der1n.getPSPickupDEField().getObjectNode().get("getRelatedPSDEField").get("name").asText();

                    LookupModel lookupModel=new LookupModel().setRelationid(der.getId())
                            .setFieldname(der1n.getPickupDEFName()).setReffieldname(relfieldname);
                    rel.addLookup(lookupModel);
                }


                entityModel.addReference(rel);
            }

        }

        if(dataEntity.getMajorPSDERs()!=null)
        {
            for(IPSDERBase der : dataEntity.getMajorPSDERs())
            {
                if(StringUtils.isEmpty(der.getMinorCodeName()))
                    continue;
                RelationshipModel rel=new RelationshipModel();
                rel.setRelation(der);
                rel.setName(der.getName()).setRelationType(der.getDERType()).setCodeName(der.getMinorCodeName()).setEntityId(der.getMinorPSDataEntity().getId())
                        .setEntityCodeName(der.getMinorPSDataEntity().getCodeName()).setEntityName(der.getMinorPSDataEntity().getName())
                        .setEntityLogicName(der.getMinorPSDataEntity().getLogicName()).setTableName(der.getMinorPSDataEntity().getTableName());


                if(der instanceof IPSDER1N)
                {
                    IPSDER1N der1n=(IPSDER1N)der;
                    String relfieldname=der1n.getPSPickupDEField().getObjectNode().get("getRelatedPSDEField").get("name").asText();

                    LookupModel lookupModel=new LookupModel().setRelationid(der.getId())
                            .setFieldname(der1n.getPickupDEFName()).setReffieldname(relfieldname);
                    rel.addLookup(lookupModel);
                }

                entityModel.addNested(rel);
            }
        }

        Map<String,FieldModel> fieldMaps=new LinkedHashMap<>();
        for(IPSDEField defield:dataEntity.getAllPSDEFields())
        {

            String dict=null;
            try { dict=defield.getPSCodeList()!=null?defield.getPSCodeList().getCodeName():null; } catch (Exception ex){}

            FieldModel fieldModel=new FieldModel();
            fieldModel.setField(defield);

            if(defield.getAllPSDEFDTColumns()!=null)
            {
                defield.getAllPSDEFDTColumns().forEach(col->{
                    if(!fieldModel.getFieldName().equalsIgnoreCase(col.getColumnName()))
                        fieldModel.set("column-"+col.getDBType().toLowerCase().replace("mysql5","mysql"),col.getColumnName());
                    if(StringUtils.isEmpty(fieldModel)&&defield.isFormulaDEField()&&(!StringUtils.isEmpty(col.getQueryCodeExp())))
                        fieldModel.setExpression(col.getQueryCodeExp().replace("`","").replace("[","").replace("]",""));
                });
            }

            if(defield.isLinkDEField() && defield instanceof IPSLinkDEField)
            {
                IPSLinkDEField linkDEField = (IPSLinkDEField)defield;
                String relfieldname=linkDEField.getObjectNode().get("getRelatedPSDEField").get("name").asText();
                String relfieldcodename=linkDEField.getObjectNode().get("getRelatedPSDEField").get("codeName").asText();
                linkDEField.getRelatedPSDEField();
                fieldModel.setRefFieldName(relfieldname).setRefFieldCodeName(relfieldcodename);

                if(!StringUtils.isEmpty(linkDEField.getPSDER().getCodeName()))
                {
                    RelationshipModel relationshipModel=entityModel.getRefMaps().get(linkDEField.getPSDER().getCodeName());
                    if(relationshipModel!=null&&(!StringUtils.isEmpty(relationshipModel.getCodeName()))) {
                        relationshipModel.addField(fieldModel);
                        fieldModel.setReference(relationshipModel);
                    }
                }

            }



            fieldMaps.put(fieldModel.getFieldName(),fieldModel);
            entityModel.addField(fieldModel);
        }



        if(dataEntity.getAllPSDEDataQueries()!=null)
        {
            dataEntity.getAllPSDEDataQueries().forEach(dataQuery->{

                try {
                    if(dataQuery.getAllPSDEDataQueryCodes()!=null)
                    {
                        dataQuery.getAllPSDEDataQueryCodes().forEach(dq->{

                            String code=getQueryCode(dq);

                            DataSetModel dsModel=new DataSetModel().setDatasetId(entityModel.getEntityName().toLowerCase()+"-dq-"+dataQuery.getCodeName()+"-"+dq.getDBType().toLowerCase().replace("mysql5","mysql"))
                                    .setDatasetName(dataQuery.getLogicName()).setCodeName(dataQuery.getCodeName())
                                    .setEntityId(entityModel.getEntityId()).setEntityName(entityModel.getEntityName()).setDsCode(code);
                            entityModel.addDataSet(dsModel);
                        });
                    }
                } catch (Exception exception) {

                }

            });
        }

        if(dataEntity.getAllPSDEDataSets()!=null)
        {
            dataEntity.getAllPSDEDataSets().forEach(dataSet->{


                try {

                    if(dataSet.getPSDEDataQueries()!=null)
                    {



                        Map<String, String> map =new HashMap<>();
                        dataSet.getPSDEDataQueries().forEach(dataQuery->{

                            try {
                                if(dataQuery.getAllPSDEDataQueryCodes()!=null)
                                {
                                    dataQuery.getAllPSDEDataQueryCodes().forEach(dq->{
                                        String code="";
                                        if(!map.containsKey(dq.getDBType())) {
                                            map.put(dq.getDBType(), "");
                                            code="";
                                        }
                                        else {
                                            code = map.get(dq.getDBType());
                                            code = code+"\r\n union all \r\n";
                                        }
                                        code = code+ getQueryCode(dq);
                                        ;//"<include refid=\""+entityModel.getEntityName().toLowerCase()+"_dq_"+dataQuery.getCodeName()+"_"+dq.getDBType().toLowerCase()+"\"/>";
                                        map.put(dq.getDBType(),"select t1.* from ("+code+") t1");
                                    });
                                }
                            } catch (Exception exception) {

                            }

                        });

                        for(Map.Entry<String, String> entry:map.entrySet())
                        {
                            String sql=null;

                            if(dataSet.getGroupMode()==1||dataSet.getMajorSortPSDEField()!=null)
                            {

                                if(dataSet.getGroupMode()==1&&(!ObjectUtils.isEmpty(dataSet.getPSDEDataSetGroupParams())))
                                {
                                    sql="select ";
                                    int i=0;
                                    for(IPSDEDataSetGroupParam obj:dataSet.getPSDEDataSetGroupParams())
                                    {
                                        if(!(obj instanceof PSDEDataSetGroupParamImpl))
                                            continue;
                                        PSDEDataSetGroupParamImpl groupParam=(PSDEDataSetGroupParamImpl)obj;
                                        if(i>0)sql=sql.concat(",");
                                        if(groupParam.isEnableGroup())
                                        {
                                            if(!StringUtils.isEmpty(groupParam.getGroupCode()))
                                                sql=sql.concat(groupParam.getGroupCode());
                                            else
                                                sql=sql.concat(groupParam.getName());
                                        }
                                        else
                                            sql=sql.concat(groupParam.getGroupCode());
                                        sql=sql.concat(" as ").concat(groupParam.getName().toLowerCase());
                                        i++;
                                    }
                                    sql=sql.concat(" from  ( %s ) t1");
                                }
                                else
                                    sql="select t1.* from ( %s ) t1";


                                if(dataSet.getGroupMode()==1&&(!ObjectUtils.isEmpty(dataSet.getPSDEDataSetGroupParams())))
                                {
                                    sql=sql.concat(" group by ");
                                    int i=0;
                                    for(IPSDEDataSetGroupParam obj:dataSet.getPSDEDataSetGroupParams())
                                    {
                                        if(!(obj instanceof PSDEDataSetGroupParamImpl))
                                            continue;
                                        PSDEDataSetGroupParamImpl groupParam=(PSDEDataSetGroupParamImpl)obj;
                                        if(groupParam.isEnableGroup())
                                        {
                                            if(i>0)sql=sql.concat(",");

                                            if(!StringUtils.isEmpty(groupParam.getGroupCode()))
                                                sql=sql.concat(groupParam.getGroupCode());
                                            else
                                                sql=sql.concat(groupParam.getName());
                                            i++;
                                        }
                                    }
                                }

                                if(dataSet.getMajorSortPSDEField()!=null)
                                {
                                    IPSDEFDTColumn column = dataSet.getMajorSortPSDEField().getPSDEFDTColumn(entry.getKey().toLowerCase().replace("mysql5","mysql"),false);

                                    sql=sql.concat(" order by ").concat(column==null?dataSet.getMajorSortPSDEField().getName():column.getColumnName());
                                    if(!StringUtils.isEmpty(dataSet.getMajorSortDir()))
                                        sql=sql.concat(" ").concat(dataSet.getMajorSortDir());
                                    if(dataSet.getMinorSortPSDEField()!=null)
                                    {
                                        IPSDEFDTColumn subCol = dataSet.getMinorSortPSDEField().getPSDEFDTColumn(entry.getKey().toLowerCase().replace("mysql5","mysql"),false);

                                        sql=sql.concat(",").concat(subCol==null?dataSet.getMinorSortPSDEField().getName():subCol.getColumnName());
                                        if(!StringUtils.isEmpty(dataSet.getMinorSortDir()))
                                            sql=sql.concat(" ").concat(dataSet.getMinorSortDir());

                                    }
                                }
                            }
                            DataSetModel dsModel=new DataSetModel().setDatasetId(entityModel.getEntityName().toLowerCase()+"-ds-"+dataSet.getCodeName()+"-"+entry.getKey().toLowerCase().replace("mysql5","mysql"))
                                    .setDatasetName(dataSet.getLogicName()).setCodeName(dataSet.getCodeName())
                                    .setEntityId(entityModel.getEntityId()).setEntityName(entityModel.getEntityName()).setDsCode(entry.getValue().toLowerCase().replace("mysql5","mysql")).setDsModel(sql);
                            entityModel.addDataSet(dsModel);
                        }

                    }
                } catch (Exception exception) {

                }

            });


        }

        entities.put(entity,entityModel);

        return entityModel;
    }




    public EntityModel getEntitySchema(String entity)
    {
        EntityModel entityModel= null;
        try {
            entityModel = getDynamicEntity(entity);
        } catch (Exception exception) {
            exception.printStackTrace();
        }
        Assert.notNull(entityModel,"loadDOModel未找到实体"+"."+entity);

        for (String dsType : entityModel.getDsTypes()) {
            POSchema poSchema = TransUtils.EntityModelModel2PO(entityModel, dsType);
            if (poSchema != null) {
                entityModel.addPOSchema(dsType, poSchema);
            }
        }
        return entityModel;
    }



    private String contextParamConvert(String contextParam)
    {
        String resultParam=getJavaSqlCode(contextParam).replaceAll("(IN|in) \\(\\$\\{srf(datacontext|sessioncontext|webcontext)\\('(\\w+)','(.*?)'\\)}\\)","$1 (\\${srf.$2.$3})");
        resultParam=resultParam.replaceAll("\\$\\{srf(datacontext|sessioncontext|webcontext)\\('(\\w+)','(.*?)'\\)}","#{srf.$1.$2}");
        resultParam=resultParam.replaceAll("\\$\\{srf(datacontext|sessioncontext|webcontext)\\('(\\w+)'\\)}","#{srf.$1.$2}");

        Pattern p = Pattern.compile("srf.(datacontext|sessioncontext|webcontext).(\\w+)");
        Matcher m = p.matcher(resultParam);
        StringBuffer sb = new StringBuffer();

        while(m.find()){
            String match = m.group();

            resultParam=resultParam.replace(match,match.toLowerCase());

        }

        resultParam=resultParam.replace("AND t11.SRFDCID = '__SRFSAASDCID__'","");
        return resultParam;
    }


    private String getQueryCode(IPSDEDataQueryCode dq)
    {
        String code=contextParamConvert(dq.getQueryCode());
        if(dq.getPSDEDataQueryCodeConds()!=null)
        {
            int i=0;
            boolean b=checkIgnoreNullvalueCond(dq.getPSDEDataQueryCodeConds());
            for(IPSDEDataQueryCodeCond cond:dq.getPSDEDataQueryCodeConds())
            {
                if(i==0)
                    code=code.concat(" where ");
                else if(i>0)
                    code=code.concat(" and ");

                code=code.concat(b?checkNullContextParamConvert(cond.getCustomCond()):contextParamConvert(cond.getCustomCond()));
                i++;
            }
        }
        return code;
    }



    private String getJavaSqlCode(String strSQLCode)
    {
        strSQLCode = strSQLCode.replace("\r\n", " ");
        strSQLCode = strSQLCode.replace("\r", " ");
        strSQLCode = strSQLCode.replace("\n", " ");
        strSQLCode = strSQLCode.replace("\"", "\\\"");
        return strSQLCode;
    }


    private String checkNullContextParamConvert(String contextParam) {
        String resultParam = getJavaSqlCode(contextParam).replaceAll("<#assign _value=srf(datacontext|sessioncontext|webcontext)\\('(\\w+)','(.*?)'\\)><#if _value\\?length gt 0>(.*?)\\(\\$\\{_value}\\)<#else>1=1</#if>"," ('${srf.$1.$2}'=null or '${srf.$1.$2}'='' or  $4  \\${srf.$1.$2} ");
        resultParam = resultParam.replaceAll("<#assign _value=srf(datacontext|sessioncontext|webcontext)\\('(\\w+)','(.*?)'\\)><#if _value\\?length gt 0>(.*?)\\$\\{_value}(.*?)?<#else>1=1</#if>", " (#{srf.$1.$2}=null or #{srf.$1.$2}='' or $4#{srf.$1.$2}$5)");
        resultParam=contextParamConvert(resultParam);
        return resultParam;
    }

    private boolean checkIgnoreNullvalueCond(List<IPSDEDataQueryCodeCond> list)
    {
        if(list!=null)
        {
            for(IPSDEDataQueryCodeCond cond:list)
            {
                if(getJavaSqlCode(cond.getCustomCond()).matches("(.*?)srf(datacontext|sessioncontext|webcontext)(.*?)\\\\\"ignoreempty\\\\\":true(.*?)"))
                    return true;
            }
        }
        return false;
    }






}
