package cn.ibizlab.codegen.model;

import cn.ibizlab.codegen.utils.DataObject;
import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.experimental.Accessors;
import net.ibizsys.model.dataentity.IPSDataEntity;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

@Getter
@Setter
@NoArgsConstructor
@Accessors(chain = true)
public class EntityModel   {

    
    
    public String getEntityId() {
        return getEntity().getId();
    }

    
    
    public String getCodeName() {
        return getEntity().getCodeName();
    }

    
    
    public String getEntityName() {
        return getEntity().getName();
    }

    
    
    public String getTableName() {
        return getEntity().getTableName();
    }

    
    
    public String getLogicName() {
        return getEntity().getLogicName();
    }

    public String getModule() {
        if(getEntity().getPSSystemModule()!=null)
            return getEntity().getPSSystemModule().getCodeName();
        return "Ungroup";
    }

    private IPSDataEntity entity;

    private List<DataSetModel> dataSets;

    public EntityModel addDataSet(DataSetModel dataSet)
    {
        if(dataSets==null)
            dataSets=new ArrayList<>();
        dataSets.add(dataSet);
        return this;
    }

    private List<FieldModel> fields;

    public EntityModel addField(FieldModel fieldModel)
    {
        if(fields==null)
            fields=new ArrayList<>();
        fields.add(fieldModel);
        return this;
    }

    private List<RelationshipModel> references;

    public EntityModel addReference(RelationshipModel relationshipModel)
    {
        if(references==null)
            references=new ArrayList<>();
        references.add(relationshipModel);
        return this;
    }

    
    
    private Map<String,RelationshipModel> refMaps;
    
    
    public Map<String,RelationshipModel> getRefMaps()
    {
        if(refMaps==null)
            refMaps=new LinkedHashMap<>();
        if(references!=null)
        {
            references.forEach(ship->{
                if(!StringUtils.isEmpty(ship.getCodeName()))
                    refMaps.put(ship.getCodeName(),ship);
                if(!StringUtils.isEmpty(ship.getRelation())) {
                    String fkname= DataObject.getStringValue(ship.getRelation().getName(),"");
                    if(!StringUtils.isEmpty(fkname))
                        refMaps.put(fkname, ship);
                }
            });
        }
        return refMaps;
    }

    private List<RelationshipModel> nesteds;

    public EntityModel addNested(RelationshipModel relationshipModel)
    {
        if(nesteds==null)
            nesteds=new ArrayList<>();
        nesteds.add(relationshipModel);
        return this;
    }

    
    
    private Map<String, FieldModel> fieldMap = null;

    
    
    public Map<String, FieldModel> getFieldMap()
    {
        if(fields!=null&&fieldMap==null)
        {
            fieldMap=new LinkedHashMap<>();
            fields.forEach(field->{
                fieldMap.put(field.getColumnName(),field);
                fieldMap.put(field.getCodeName(),field);
            });
        }
        return fieldMap;
    }

    
    
    private FieldModel lastModifyField;
    
    
    public FieldModel getLastModifyField() {
        if(fields!=null&&lastModifyField==null)
            for(FieldModel fieldModel:fields)
                if(fieldModel.isLastModifyField())
				{
					lastModifyField=fieldModel;
					return lastModifyField;
				}
        return lastModifyField;
    }

	
	
    private boolean isLogicValid=true;

    
    
    public boolean isLogicValid()
    {
        if(isLogicValid&&logicValidField==null) {
            if (fields != null) {
                for (FieldModel fieldModel : fields) {
                    if (fieldModel.isLogicValidField()) {
                        logicValidField = fieldModel;
                        return isLogicValid;
                    }
                }
            }
            isLogicValid = false;
        }
        return isLogicValid;
    }
	
	
    private FieldModel logicValidField;
	
	
	public FieldModel getLogicValidField() {
		if(isLogicValid&&logicValidField==null) {
			if (fields != null) {
				for (FieldModel fieldModel : fields) {
					if (fieldModel.isLogicValidField()) {
						logicValidField = fieldModel;
						return logicValidField;
					}
				}
			}
			isLogicValid = false;
		}
		return logicValidField;
	}

	
	
	private String validLogicValue;
	
	
	public String getValidLogicValue()
	{
		if(StringUtils.isEmpty(validLogicValue))
            validLogicValue=this.getEntity().getValidLogicValue();
		if(StringUtils.isEmpty(validLogicValue))
            validLogicValue="1";
		return validLogicValue;
	}
	
	
	private String invalidLogicValue;
	
	
	public String getInvalidLogicValue()
	{
		if(StringUtils.isEmpty(invalidLogicValue))
            invalidLogicValue=this.getEntity().getInvalidLogicValue();
		if(StringUtils.isEmpty(invalidLogicValue))
            invalidLogicValue="0";
		return invalidLogicValue;
	}
	
	
	private FieldModel keyField;
    
    
    public FieldModel getKeyField() {
        if(fields!=null&&keyField==null)
            for(FieldModel fieldModel:fields)
                if(fieldModel.isKeyField()) {
					keyField=fieldModel;
					return keyField;
				}
        return keyField;
    }

	
	
    private List<FieldModel> unionKeyFields;
	
	
	public List<FieldModel> getUnionKeyFields() {
		if(fields!=null&&unionKeyFields==null) {
			unionKeyFields = new ArrayList<>();
			for (FieldModel fieldModel : fields)
				if (fieldModel.isUnionKeyField())
					unionKeyFields.add(fieldModel);
		}
		return unionKeyFields;
	}

    
    
    public List<FieldModel> getKeyFields() {
        if(this.getKeyField()!=null&&this.getKeyField().isPhysicalField()) {
            List<FieldModel> keyFields = new ArrayList<>();
            keyFields.add(getKeyField());
            return keyFields;
        }
        else
            return getUnionKeyFields();
    }


    
    
    public String getDsName()
    {
        String dsName=this.getEntity().getDSLink();
        if(StringUtils.isEmpty(dsName)||"DEFAULT".equalsIgnoreCase(dsName))
        {
            dsName="master";
        }
        return dsName;
    }



    public String getTableName(String dsType)
    {
        return this.getStringValue("table-"+dsType.toLowerCase(),getTableName());
    }

    
    
    public List<String> getDsTypes()
    {
        List<String> dsTypes=new ArrayList<>();
        String strDsTypes=this.getStringValue("ds_types","");
        for(String dsType:strDsTypes.split(","))
            dsTypes.add(dsType);
        return dsTypes;
    }



	private String storageMode="SQL";

	public void setStorageMode(Integer type)
    {
        switch(type){
            case 0:
                this.storageMode="NONE";
                break;
            case 1:
                this.storageMode="SQL";
                break;
            case 2:
                this.storageMode="NoSQL";
                break;
            case 4:
                this.storageMode="ServiceAPI";
                break;
            default:
                this.storageMode="SQL";
                break;
        }
    }
    public void setStorageMode(String type)
    {
        this.storageMode=type;
    }



    @JSONField(serialize = false)
    @JsonIgnore
    private Map<String, POSchema> poSchemas;

    @JSONField(serialize = false)
    @JsonIgnore
    public POSchema getDefaultPOSchema()
    {
        return getPOSchema("default");
    }
    public EntityModel addPOSchema(String name, POSchema poSchema)
    {
        if(poSchema!=null)
        {
            if(poSchemas==null)
                poSchemas=new LinkedHashMap<>();
            poSchemas.put(name,poSchema.build());
        }

        return this;
    }
    public POSchema getPOSchema(String name)
    {

        if(StringUtils.isEmpty(name)&&(!StringUtils.isEmpty(this.getDsName())))
            name=this.getDsName();
        if(StringUtils.isEmpty(name))
            name="mysql";

        if(poSchemas==null)
            poSchemas=new LinkedHashMap<>();
        if(poSchemas.containsKey(name))
        {
            return poSchemas.get(name);
        }
        String vendorProvider=POSchema.provider.get(name.toLowerCase());
        if((!StringUtils.isEmpty(vendorProvider))&&(!name.equalsIgnoreCase(vendorProvider))&&poSchemas.containsKey(vendorProvider))
            return poSchemas.get(vendorProvider);

        if(poSchemas.size()>0)
            return poSchemas.values().iterator().next();

        return null;
    }

    private DataObj extParams=new DataObj();

    public EntityModel set(String key,Object val)
    {
        extParams.set(key,val);
        return this;
    }

    public Object get(String key)
    {
        return key;
    }

    public String getStringValue(String key,String defaultVal)
    {
        return extParams.getStringValue(key,defaultVal);
    }
}
