package cn.ibizlab.codegen.model;

import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.experimental.Accessors;
import net.ibizsys.model.dataentity.defield.IPSDEField;
import org.springframework.util.StringUtils;

import java.util.LinkedHashSet;
import java.util.Set;

@Getter
@Setter
@NoArgsConstructor
@Accessors(chain = true)
@JsonIgnoreProperties(value = "handler")
public class FieldModel  {

    public String getCodeName() {
        return getField().getCodeName();
    }



    public String getFieldName() {
        return getField().getName();
    }

    public String getColumnName(){
        return getField().getName();
    }


    public String getLogicName() {
        return getField().getLogicName();
    }

    public DataType getType()
    {
        return DataType.findTypeName(this.getField().getStdDataType());
    }

    private String expression;

    private IPSDEField field;

    private RelationshipModel reference;

    private String refFieldName;

    private String refFieldCodeName;



    @JsonIgnore
    @JSONField(serialize = false)
    public boolean isPhysicalField()
    {
        return this.getField().isPhisicalDEField();
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public boolean isKeyField()
    {
        return this.getField().isKeyDEField();
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public boolean isLogicValidField()
    {
        return "LOGICVALID".equals(this.getField().getPredefinedType());
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public boolean isLastModifyField()
    {
        return "UPDATEDATE".equals(this.getField().getPredefinedType());
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public boolean isCreateTimeField()
    {
        return "CREATEDATE".equals(this.getField().getPredefinedType());
    }

    @JsonIgnore
    @JSONField(serialize = false)
    public boolean isUnionKeyField()
    {
        return !StringUtils.isEmpty(this.getField().getUnionKeyValue());
    }

    public String getDefaultValue()
    {
        if((!StringUtils.isEmpty(getField().getDefaultValueType()))||(!StringUtils.isEmpty(getField().getDefaultValue())))
        {
            String defaultValue="";
            if(StringUtils.isEmpty(getField().getDefaultValueType())||"NONE".equalsIgnoreCase(getField().getDefaultValueType()))
            {
                defaultValue=getField().getDefaultValue();
            }
            else if(!StringUtils.isEmpty(getField().getDefaultValue()))
            {
                defaultValue= String.format("${%1$s.%2$s}",getField().getDefaultValueType(),getField().getDefaultValue());
            }
            else
            {
                defaultValue= String.format("${%1$s}",getField().getDefaultValueType());
            }
            return defaultValue;
        }
        return null;
    }

    public Set<String> getSearchMode()
    {
        Set<String> modes=new LinkedHashSet<>();
        if(getField().getAllPSDEFSearchModes()!=null)
        {
            getField().getAllPSDEFSearchModes().forEach(item->modes.add(item.getValueOP().toLowerCase()));
        }
        if(getField().isEnableQuickSearch())
            modes.add("query");
        return modes;
    }


    private DataObj extParams=new DataObj();

    public FieldModel set(String key,Object val)
    {
        extParams.set(key,val);
        return this;
    }

    public String getStringValue(String key,String defaultVal)
    {
        return extParams.getStringValue(key,defaultVal);
    }

    public String getColumnName(String dsType)
    {
        return this.getStringValue("column-"+dsType.toLowerCase(),getColumnName());
    }

    public static enum  DataType {
        UNKNOWN(0,"UNKNOWN","string","String"),
        BIGINT(1,"BIGINT","number","Long"),
        BINARY(2,"BINARY","string","String"),
        BIT(3,"BIT","boolean","Boolean"),
        CHAR(4,"CHAR","string","String"),
        DATETIME(5,"DATETIME","string", "Timestamp"),
        DECIMAL(6,"DECIMAL","number", "BigDecimal"),
        FLOAT(7,"FLOAT","number", "BigDecimal"),
        IMAGE(8,"IMAGE","string","String"),
        INT(9,"INT","number","Integer"),
        MONEY(10,"MONEY","number", "BigDecimal"),
        NCHAR(11,"NCHAR","string","String"),
        NTEXT(12,"NTEXT","string","String"),
        NVARCHAR(13,"NVARCHAR","string","String"),
        NUMERIC(14,"NUMERIC","number", "BigDecimal"),
        REAL(15,"REAL","number","Float"),
        SMALLDATETIME(16,"SMALLDATETIME","string", "Timestamp"),
        SMALLINT(17,"SMALLINT","number","Integer"),
        SMALLMONEY(18,"SMALLMONEY","number", "BigDecimal"),
        SQL_VARIANT(19,"SQL_VARIANT","string","String"),
        SYSNAME(20,"SYSNAME","string","String"),
        TEXT(21,"TEXT","string","String"),
        TIMESTAMP(22,"TIMESTAMP","string","Timestamp"),
        TINYINT(23,"TINYINT","number","Integer"),
        VARBINARY(24,"VARBINARY","string","String"),
        VARCHAR(25,"VARCHAR","string","String"),
        UNIQUEIDENTIFIER(26,"UNIQUEIDENTIFIER","string","String"),
        DATE(27,"DATE","string", "Timestamp"),
        TIME(28,"TIME","string", "Timestamp");

        public final int code;
        public final String name;
        public final String prop;
        public final String java;

        private DataType(int code, String name, String prop, String java) {
            this.code = code;
            this.name = name;
            this.prop = prop;
            this.java = java;
        }

        public int getCode() {
            return code;
        }

        public String getName() {
            return name;
        }

        public String getProp() {
            return prop;
        }

        public String getJava() {
            return java;
        }

        public static DataType findTypeName(Integer type) {
            for (DataType userTypeEnum : DataType.values()) {
                if (userTypeEnum.getCode()==type) {
                    return userTypeEnum;
                }
            }
            return VARCHAR;
        }

    }




}
