package cn.ibizlab.codegen.model;


import cn.ibizlab.codegen.CodegenConfig;
import cn.ibizlab.codegen.CodegenConstants;

import cn.ibizlab.codegen.templating.TemplateFileType;
import net.ibizsys.model.IPSSystem;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;

import java.nio.file.Paths;
import java.util.*;


public class ModelStorage {

    public static String USER_HOME = Paths.get(System.getProperty("user.home"),".ibizlab").toString();


    private static ModelStorage instance=null;

    public static ModelStorage getInstance()
    {
        if(instance==null)
            instance=new ModelStorage();
        return instance;
    }

    private CodegenConfig config;

    public ModelStorage config(CodegenConfig config)
    {
        this.config=config;
        DynamicModelStorage.getInstance().modelPath(config.getInputSpec());
        return this;
    }

    private DynamicModelStorage dynamicService=DynamicModelStorage.getInstance();



    public CliOption newCliOption(TemplateFileType type)
    {
        CliOption opt=new CliOption();
        opt.putAll(config.getAdditionalProperties());
        opt.setTemplateFileType(type);

        IPSSystem ipsSystem=dynamicService.getPSSystem();
        if(StringUtils.isEmpty(config.getAdditionalProperties().get(CodegenConstants.PROJECT_NAME)))
            opt.setProjectName(ipsSystem.getCodeName().toLowerCase()).toString();
        if(StringUtils.isEmpty(config.getAdditionalProperties().get(CodegenConstants.PROJECT_DESC)))
            opt.setProjectDesc(ipsSystem.getName().toLowerCase()).toString();
        if(StringUtils.isEmpty(config.getAdditionalProperties().get(CodegenConstants.PACKAGE_NAME)))
            opt.setPackageName(ipsSystem.getCodeName().toLowerCase()).toString();
        opt.set("system",ipsSystem);
        return opt;
    }




    private Map<TemplateFileType,CliData> templateData=new HashMap<>();

    public CliData getTemplateData(TemplateFileType type)
    {
        if(!templateData.containsKey(type))
        {
            CliData rt=new CliData();
            if(type.equals(TemplateFileType.api))
            {
                if(dynamicService.getPSSystem().getAllPSSysServiceAPIs()!=null)
                {
                    dynamicService.getPSSystem().getAllPSSysServiceAPIs().forEach(item->{
                        CliOption opt=newCliOption(TemplateFileType.api).baseData(item,item.getCodeName().toLowerCase());
                        rt.addOption(opt);
                    });
                }
            }
            else if(type.equals(TemplateFileType.app))
            {
                if(dynamicService.getPSSystem().getAllPSApps()!=null)
                {
                    dynamicService.getPSSystem().getAllPSApps().forEach(item->{
                        CliOption opt=newCliOption(TemplateFileType.app).baseData(item,item.getCodeName().toLowerCase());
                        rt.addOption(opt);
                    });
                }
            }
            else if(type.equals(TemplateFileType.serviceApi))
            {
                dynamicService.getPSSystem().getAllPSSysServiceAPIs().forEach(api->{
                    if(api.getPSDEServiceAPIs()!=null)
                    {
                        api.getPSDEServiceAPIs().forEach(item->{
                            CliOption opt=newCliOption(TemplateFileType.serviceApi).baseData(item,item.getCodeName());
                            rt.addOption(opt);
                        });
                    }
                });
            }
            else if(type.equals(TemplateFileType.entity))
            {
                dynamicService.getPSSystem().getAllPSDataEntities().forEach(item->{
                    EntityModel schema=dynamicService.getEntitySchema(item.getId());
                    CliOption opt=newCliOption(TemplateFileType.entity).setCliSubType(schema.getStorageMode()).setModule(schema.getModule())
                            .baseData(schema,item.getCodeName());
                    rt.addOption(opt);
                });
            }
            else if(type.equals(TemplateFileType.module))
            {
                dynamicService.getPSSystem().getAllPSSystemModules().forEach(item->{
                    CliOption opt=newCliOption(TemplateFileType.module)
                            .baseData(item,item.getCodeName());
                    rt.addOption(opt);
                });
            }
            else if(type.equals(TemplateFileType.supportingFiles))
            {
                CliOption opt=newCliOption(TemplateFileType.supportingFiles)
                        .set(TemplateFileType.app.value(),getTemplateData(TemplateFileType.app))
                        .set(TemplateFileType.api.value(),getTemplateData(TemplateFileType.api))
                        .set(TemplateFileType.module.value(),getTemplateData(TemplateFileType.module));

                rt.addOption(opt);
            }
            templateData.put(type,rt);
        }
        return templateData.get(type);
    }



}
